package de.unikoblenz.fgbks.base.value;

import static java.util.Objects.nonNull;

import java.io.Serializable;
import java.util.Arrays;
import java.util.stream.Collectors;

public abstract class AbstractValueObject implements Serializable, ValueObject {

  private transient Object[] values;

  @Override
  public int hashCode() {
    return Arrays.hashCode(getValues());
  }

  @Override
  public boolean equals(Object object) {
    if (this == object) {
      return true;
    }
    if (object == null || !getClass().isAssignableFrom(object.getClass())) {
      return false;
    }
    AbstractValueObject valueObject = (AbstractValueObject) object;
    return Arrays.equals(getValues(), valueObject.getValues());
  }

  public boolean isAllNull() {
    return !Arrays.stream(getValues()).anyMatch(v -> nonNull(v));
  }

  /** Returns all values of this value object */
  protected abstract Object[] values();

  private Object[] getValues() {
    if (values == null) {
      values = values();
    }
    return values;
  }

  @Override
  public String toString() {
    return "["
        + Arrays.stream(getValues())
            .map(o -> nonNull(o) ? o.toString() : "null")
            .collect(Collectors.joining(", "))
        + "]";
  }

  /**
   * Value kann null sein wenn durch JPA geladen wurde Spalte nicht null ist aber als null
   * interpretiert werden soll
   */
  public boolean nonNullValue() {
    return true;
  }
}
